// ***********************************************
// This is the source of an sample implementation
// of the E-LAB Mega8 Terminal
// The Terminal is connected to the EvaBoardI
// through a TWI Master/Slave connection
// where this implementation is the Master on EvaBoard
// ***********************************************

program EVATerm8Demo;

{ $BOOTRST $0F000}         {Reset Jump to $0F000}
{ $W+ Warnings}            {Warnings off}

Device = mega128, VCC = 5;

Import SysTick, SerPort, TWImaster;

From System Import ;

Define
  ProcClock      = 16000000;        {Hertz}
  SysTick        = 10;             {msec}
  StackSize      = $0064, iData;
  FrameSize      = $0064, iData;
  SerPort        = 19200, Stop2;    {Baud, StopBits|Parity}
  RxBuffer       = 8, iData;
  TxBuffer       = 40, iData;
  TWIpresc       = TWI_BR400;

Implementation

{$IDATA}

{--------------------------------------------------------------}
{ Type Declarations }
type
  tTermCmd       = (BusyState, KeybState, DispCmd, SoundCmd, KeybFlush, BarInit);

  tLCcmd         = (lcClr, lcHome, lcXY, lcStr, lcClrLn, lcClrEOL, lcBar, lcCurs);
  tLCcmdSet      = BitSet of tLCcmd;

  tBusy          = (lcBusy, bpBusy);
  tBusyState     = set of tBusy;

  tTermState     = record
                     BusyState : tBusyState;
                     KeyStates : word;
                   end;


  t_Display      = Record
                     Cmd            : tLCcmdSet;
                     x              : Byte;
                     y              : Byte;
                     Str            : String[20];
                   end;

  tBeepCmd       = (bpOff, bpErr, bpClick, bpLH, bpHL, bpSir0, bpSir1, bpOut);
  t_Sound        = Record
                     cmd            : tBeepCmd;
                     Frequ          : Word;
                     Ticks          : Byte;
                   end;

  t_Bar          = Record
                     Cmd            : tLCcmdSet;
                     Value          : Byte;
                   end;

  t_BarIni       = Record
                     x              : Byte;
                     y              : Byte;
                     Len            : Byte;
                     Scal           : Byte;
                   end;

  tKeys          = (Key3, Key9, Key6, KeyEnter, Key2, Key8,
                    Key5, KeyShift, Key1, Key7, Key4, Key0);
  tKeySet        = set of tKeys;
  
{--------------------------------------------------------------}
{ Const Declarations }
Const
  Disp_Clear     : Byte      = %00000001;          // Clears the Display //
  Disp_Home      : Byte      =  %00000010;         // Cursor Home //
  Disp_CurPos    : Byte      = %00000100;          // Write Command //
  Disp_Write     : Byte      = %00001000;          // Cursor ON/Off //
  Disp_CurOn     : Byte      = %01000000;          // Cursor Bliobk //
  Disp_CurBl     : Byte      = %10000000;

{--------------------------------------------------------------}
{ Var Declarations }
{$IDATA}
Var
  Keys, prevKeys  : word;
  KeySet[@Keys]   : tKeySet;
  shift           : boolean;
  st              : string[30];
  doGraph         : boolean;
  doErr           : boolean;

{--------------------------------------------------------------}
{ functions }
Procedure Term_Display(Disp_cmd : tLCcmdSet; Pos_x, Pos_y : Byte; Out_str : String[20]);
Var
  Buffer  : t_Display;
Begin
  Buffer.Cmd:= Disp_cmd;
  Buffer.x:= Pos_x;
  Buffer.y:= Pos_Y;
  Buffer.str:= Out_str;
  Repeat
  Until Twiout(02, DispCmd, Buffer);
End;

Procedure Term_BarOut(Value : Byte);
Var
  Buffer  : T_Bar;
Begin
  Buffer.cmd := [lcBar];
  Buffer.Value := Value;
  Repeat
  Until Twiout(02, DispCmd, Buffer);
end;

Procedure Term_BarInit(Pos_X : Byte; Pos_Y : Byte; Len : Byte; Scal : Byte);
Var
  Buffer  : t_BarIni;
Begin
  Buffer.x:= Pos_x;
  Buffer.y:= Pos_Y;
  Buffer.Len := Len;
  Buffer.Scal := Scal;
  Repeat
  Until Twiout(02, BarInit, Buffer);
End;


Procedure Term_Sound(Sound_Cmd : tBeepCmd; Sound_Frequ : Word; Sound_Ticks : Byte);
Var
  Buffer  : t_sound;
Begin
  Buffer.Cmd := Sound_Cmd;
  Buffer.Frequ := Sound_Frequ;
  Buffer.Ticks := Sound_Ticks;
  Repeat
  Until TWIOut(02, SoundCmd, Buffer);
End;

Procedure Term_KeybFlush;
Begin
  Repeat
  Until TWIOut(02, byte(KeybFlush));
end;

Function Term_ReadKey : word;
Var
  RdVal  : tTermState;
Begin
  Repeat
  Until TWIOut(02, KeybState);
  Repeat
  Until TWIInp(02, RdVal);
  Return(RdVal.KeyStates);
end;

Function Term_Busy : tBusyState;
Var
  RdVal  : tTermState;
Begin
  Repeat
  Until TWIOut(02, BusyState);
  Repeat
  Until TWIInp(02, RdVal);
  Return(RdVal.BusyState);
end;


procedure PlayMusic(var s : string[30]);
var
  x              : byte;
  octave         : word;
begin
  octave:= 2;
  for x:= 1 to length(s) do
    case s[x] of
      'C' : Term_Sound(bpOut, 262*octave, 100);
          |
      'd' : Term_Sound(bpOut, 262*octave, 100);
          |
      'D' : Term_Sound(bpOut, 294*octave, 100);
          |
      'e' : Term_Sound(bpOut, 311*octave, 100);
          |
      'E' : Term_Sound(bpOut, 330*octave, 100);
          |
      'F' : Term_Sound(bpOut, 349*octave, 100);
          |
      'g' : Term_Sound(bpOut, 370*octave, 100);
          |
      'G' : Term_Sound(bpOut, 392*octave, 100);
          |
      'a' : Term_Sound(bpOut, 415*octave, 100);
          |
      'A' : Term_Sound(bpOut, 440*octave, 100);
          |
      'b' : Term_Sound(bpOut, 446*octave, 100);
          |
      'B' : Term_Sound(bpOut, 494*octave, 100);
          |
      's' : Term_Sound(bpOut, 0*octave, 100);
          |
      'S' : Term_Sound(bpOut, 0*octave, 255);
          |
    endcase;
  endfor;
end;

Procedure Show_Graph;
Var
  i  : Byte;
Begin
  for i := 0 to 200 Do
    Term_BarOut(i);
    Mdelay(10);
  End_For;
  for i := 200 downto 0 Do
    Term_BarOut(i);
    Mdelay(10);
  End_For;
end;


{--------------------------------------------------------------}
{ Main Program }
{$IDATA}

begin
  Mdelay(100);
  Start_Processes;
  Term_BarInit(2, 0, 20, 200);
  Term_Display([lcCurs], byte(true), byte(true), '');
  Term_Display([lcClr, lcXY, lcStr], 0, 0, 'E-LAB Computers');
  Term_Display([lcXY, lcStr], 0, 1, 'Mega8 Terminal');
  Term_Display([lcXY, lcStr], 0, 2, 'Demo Program');
  repeat
  until Term_Busy = [];
  Term_Sound(bpErr, 0, 0);
  repeat
  until Term_Busy = [];
  Mdelay(200);
  Term_Sound(bpClick, 0, 0);
  repeat
  until Term_Busy = [];
  Mdelay(200);
  Term_Sound(bpLH, 0, 0);
  repeat
  until Term_Busy = [];
  Mdelay(200);
  Term_Sound(bpHL, 0, 0);
  repeat
  until Term_Busy = [];
  Mdelay(200);
  Term_Sound(bpSir0, 0, 2);
  repeat
  until Term_Busy = [];
  Mdelay(200);
  Term_Sound(bpSir1, 0, 2);
  repeat
  until Term_Busy = [];
  Term_Display([lcClr, lcXY, lcStr], 0, 0, 'I Play a Song,');
  Term_Display([lcXY, lcStr], 0, 1, 'just a for You!!');
  Term_Display([lcXY, lcStr], 0, 2, 'It is in the Buffer');
  Term_Display([lcXY, lcStr], 0, 3, 'of the Terminal!!');
  repeat
  until Term_Busy = [];
  Mdelay(200);
  st:= 'EsEsESEsEsESEsGsCsDsESFsFsFSEsEsESDsDsDsEsDsGSEsEsESEsEsESEsG';
  PlayMusic(st);
  repeat
  until Term_Busy = [];
  Term_Display([lcClr, lcXY, lcStr], 0, 0, 'E-LAB Computers');
  Term_Display([lcXY, lcStr], 1, 1, 'E-LAB Computers');
  Term_Display([lcXY, lcStr], 2, 2, 'E-LAB Computers');
  Term_Display([lcXY, lcStr], 3, 3, 'E-LAB Computers');
  repeat
  until Term_Busy = [];
  Mdelay(2000);
  Term_Display([lcClr, lcXY, lcStr], 0, 0, 'Keyboard Test:');
  Term_Display([lcCurs], byte(false), byte(false), '');
  loop
    prevKeys:= keys;
    keys: = Term_ReadKey;
    Term_Display([lcXY, lcStr], 0, 3, 'Scancode: ' + IntToHex(Keys));

    if (keys <> prevKeys) then
      Term_Sound(bpClick, 0, 0);
      doGraph:= false;
      doErr:= false;
      if [KeyShift] = KeySet then
        st:= 'Shift';
        shift:= true;
      elsif KeyShift in KeySet then
        st:= 'Shift + ';
        shift:= true;
      else
        st:= '';
        shift:= false;
      endif;
      if Key1 in KeySet then
        Append('Key1', st);
      elsif Key2 in KeySet then
        Append('Key2', st);
      elsif Key3 in KeySet then
        Append('Key3', st);
      elsif Key4 in KeySet then
        Append('Key4', st);
      elsif Key5 in KeySet then
        Append('Key5', st);
      elsif Key6 in KeySet then
        Append('Key6', st);
      elsif Key7 in KeySet then
        Append('Key7', st);
      elsif Key8 in KeySet then
        Append('Key8', st);
      elsif Key9 in KeySet then
        Append('Key9', st);
      elsif Key0 in KeySet then
        Append('Key0', st);
        if Shift then
          doGraph:= true;
        endif;
      elsif KeyEnter in KeySet then
        Append('Enter', st);
        if Shift then
          doErr:= true;
        endif;
      endif;
      Term_Display([lcXY, lcStr], 0, 1, st);
      Term_Display([lcClrEOL], 0, 0, '');
      repeat
      until Term_Busy = [];
      if doGraph then
        Show_Graph;
      endif;
      if doErr then
        Term_Sound(bpErr, 0, 0);
      endif;
    endif;
    Mdelay(200);
  endloop;
end EVATerm8Demo.


